<?php
class RestaurantsController extends AppController {

	var $name = 'Restaurants';
	var $components = array('RequestHandler');	
	var $helpers = array('Html', 'Form', 'Text', 'Tagcloud');
    var $paginate = array(
        //'fields' => 'DISTINCT *'
	    );
	
	function beforeFilter() {
		parent::beforeFilter();	
		$this->Auth->allow(array('view', 'inaccurate', 'tag', 'area', 'cuisine', 'nearby', 'vegetarian', 'vegan', 'rate', 'glutenfree', 'home', 'id2slug', 'location'));
		$this->paginate['Restaurant']['limit'] = 10;				
	} // beforeFilter	

	function index() {
		$this->Restaurant->recursive = 0;
		$this->set('restaurants', $this->paginate());
	}
	
	function home() {
		$this->set('restaurants', $this->Restaurant->find('random'));
	}

	function view($slug = null) {
		if (!$slug) {
			$this->Session->setFlash(__('Invalid Restaurant', true));
		}
		$this->set('restaurant', $this->Restaurant->findBySlug($slug));
	}
	
	function edit($id = null) {
		if(($id != $this->me['User']['restaurant_id']) && !$this->me['User']['admin']) :
			$this->Session->setFlash('You may not edit restaurants other than your own.', true);
			$this->redirect(array('controller' => 'pages', 'action' => 'display', 'home'));
		endif;
		
		if (!$id && empty($this->data)) {
			$this->Session->setFlash(__('Invalid Restaurant', true));
			exit();
		}
		if (!empty($this->data)) {
			if ($this->Restaurant->save($this->data)) {
				$this->Session->setFlash(__('The Restaurant has been saved.', true));
			} else {
			}
		}
		if (empty($this->data)) {
			$this->data = $this->Restaurant->read(null, $id);
		}
		
	# set a blank tag list, one from this->data, or one from the restaurants tags
		if((!isset($this->data['Restaurant']['taglist'])) && ((!isset($this->data['Tag'])) || (sizeof($this->data['Tag']) <= 0))) :
			$taglist = '';
		elseif(isset($this->data['Restaurant']['taglist'])) :
			$taglist = $this->data['Restaurant']['taglist'];
		else :
			$temptags = array();
			foreach($this->data['Tag'] as $tag) :
				$temptags[] = $tag['name'];
			endforeach;
			$taglist = implode(', ', $temptags);
		endif;
		
	# current picture	
		$currentPicture = $this->Restaurant->Picture->find('first', array('conditions' => 'Picture.restaurant_id = ' . $this->data['Restaurant']['id']));
		
	# current menu
		$this->Menu->recursive = -1;
		$currentMenu = $this->Restaurant->Menu->find('first', array('conditions' => 'Menu.restaurant_id = ' . $this->data['Restaurant']['id']));
		
	# all tags w/ weights
		$tags = $this->Tag->weights();	
	
		$cuisinelist = $this->Cuisine->find('list');
		$arealist = $this->Area->find('list');
		$reviewlist = array();
		$this->Restaurant->Review->recursive = -1;
		$reviewlist = $this->Restaurant->Review->find('all', array('conditions' => 'Review.restaurant_id = ' . $this->data['Restaurant']['id']));
		$no_search = true;		
		$this->set('no_search', true);
		$this->set('restaurant_id', $this->data['Restaurant']['id']);
		$this->set(compact('tags', 'taglist', 'currentPicture', 'cuisinelist', 'arealist', 'currentMenu', 'reviewlist'));
	} // edit
	
	function inaccurate($slug = null) {
		if (!$slug) {
			$this->Session->setFlash(__('Invalid Restaurant', true));
		}
		$restaurant = $this->Restaurant->findBySlug($slug);
		$this->Restaurant->doIncrement($restaurant['Restaurant']['id']); 
		die();
	} // inaccurate
	
	function rate($slug = null, $rating = null) {
		if (!$slug || !$rating) {
			$this->Session->setFlash(__('Invalid id', true));
		}
		
		$this->Restaurant->recursive = -1;
		$restaurant = $this->Restaurant->findBySlug($slug);
		$this->Restaurant->doIncrement($restaurant['Restaurant']['id'], 1, 'rating_votes');
		$this->Restaurant->doIncrement($restaurant['Restaurant']['id'], $rating, 'rating_sum');
		$restaurant = $this->Restaurant->findBySlug($slug);

		if($restaurant['Restaurant']['rating'] == 0) :
			$restaurant['Restaurant']['rating'] = $rating;	
		else :
			$restaurant['Restaurant']['rating'] = $restaurant['Restaurant']['rating_sum'] / $restaurant['Restaurant']['rating_votes'];		
		endif;

		$data['Restaurant']['id'] = $restaurant['Restaurant']['id'];
		$data['Restaurant']['rating'] = $restaurant['Restaurant']['rating'];
		$this->Restaurant->validate = array();
		$this->Restaurant->save($data);
		die();
	}
	
	function tag($tag) {
		if(!$tag) {
			$this->Session->setFlash(__('Invalid tag', true));			
		}
		
		if($this->Tag->findBySlug($tag)) :
			if($this->layoutPath == 'json') :
				$this->set('restaurants', $this->Restaurant->find('tag', array('tag' => $tag)));
			else :		
				$ids = $this->Restaurant->find('tag', array('tag' => $tag, 'forPagination' => true));
				if($ids) :
					$ids = implode(', ', $ids);
					$where = 'Restaurant.id IN (' . $ids . ')';		
					$this->set('restaurants', $this->paginate('Restaurant', $where));
				else : 
					$this->set('restaurants', array());
				endif;
			endif;
		else :
			$this->set('restaurants', array());		
		endif;
	}
	
	function area($area) {
		if(!$area) {
			$this->Session->setFlash(__('Invalid area', true));	
			$this->redirect('/');		
		}
		if($this->layoutPath == 'json') :
			$this->set('restaurants', $this->Restaurant->find('area', array('area' => $area)));
		else :		
			$ids = $this->Restaurant->find('area', array('area' => $area, 'forPagination' => true));
			if($ids) :
				$ids = implode(', ', $ids);
				$where = 'Restaurant.id IN (' . $ids . ')';		
				$this->set('restaurants', $this->paginate('Restaurant', $where));
			else : 
				$this->set('restaurants', array());
			endif;
		endif;
	}

	function cuisine($cuisine) {
		if(!$cuisine) {
			$this->Session->setFlash(__('Invalid cuisine', true));	
			$this->redirect('/');		
		}
		
		if($this->Cuisine->findBySlug($cuisine)) :
			if($this->layoutPath == 'json') :
				$this->set('restaurants', $this->Restaurant->find('cuisine', array('cuisine' => $cuisine)));
			else :		
				$ids = $this->Restaurant->find('cuisine', array('cuisine' => $cuisine, 'forPagination' => true));
				if($ids) :
					$ids = implode(', ', $ids);
					$where = 'Restaurant.id IN (' . $ids . ')';		
					$this->set('restaurants', $this->paginate('Restaurant', $where));
				else : 
					$this->set('restaurants', array());
				endif;
			endif;
		else :
			$this->set('restaurants', array());
		endif;
	}	
	
	function latlon($lat = null, $lon = null, $radius = .3) {
		if($lat && $lon) :
			if($this->layoutPath == 'json') :
				$this->set('restaurants', $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius)));					
			else :		
				$ids = $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius, 'forPagination' => true));
				if($ids) :
					$ids = implode(', ', $ids);
					$where = 'Restaurant.id IN (' . $ids . ')';		
					$this->set('restaurants', $this->paginate('Restaurant', $where));
				else : 
					$this->set('restaurants', array());
				endif;
			endif;
		endif; 
	}
	
	# if you have a locator device
	function location() {	
		$lat = $this->params['named']['lat'];
		$lon = $this->params['named']['lon'];
		$radius = $this->params['named']['radius'];
		$this->set('no_search', true);		
		
		if($this->layoutPath == 'json') :
			$this->set('restaurants', $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius)));					
		else :		
			$ids = $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius, 'forPagination' => true));
			if($ids) :
				$ids = implode(', ', $ids);
				$where = 'Restaurant.id IN (' . $ids . ')';		
				$this->set('restaurants', $this->paginate('Restaurant', $where));
			else : 
				$this->set('restaurants', array());
			endif;
		endif;	
	}

	function nearby() {
		//$this->set('restaurants', $this->params);
		if(!empty($this->data['el'])) :
			$this->params['named']['el'] = $this->data['el'];
		endif;		
	
		if(!empty($_POST['el'])) :
			$this->params['named']['el'] = $_POST['el'];		
		endif;
	
		if(!isset($this->params['named']['el']) || $this->params['named']['el'] == '') $this->params['named']['el'] = ' ';

		if(!empty($this->params['named']['el'])) :
		# sanitize the data
			$s = str_replace(',', '', $this->params['named']['el']);
			App::import('Sanitize');			
			$s = Sanitize::clean($s);
	
		# if your string doesn't contain "richmond" add it
			if(!strstr(strtolower($s), 'richmond'))
				$s = $s . ' richmond va';
			
		
		# geocode the address	
			$parameters = array(
				'key' => 'ABQIAAAAvBCyD5zZKcF1SS9PddFlExTjLRVDPsUQGnTil05vhdPsFARv5xTJ-K_EvkdjNPsNBS0QZJN-QkihcA', 
				'q' => $s,
				'sensor' => true,
				'output' => 'json'
			);
			$parameters = http_build_query($parameters);
			$url = 'http://maps.google.com/maps/geo?' . $parameters;
			$data = file_get_contents($url);
			$data = json_decode($data);			
		
			$this->set('restaurants', array());
		
			if($data->Status->code == 200) :
			# if you've got only one address
				if(sizeof($data->Placemark) != 1) :
					$this->Session->setFlash('Your address could not be found, double check and try again.', true, array('class' => 'error'));
			#you've got good data
				else :
					$lat = $data->Placemark[0]->Point->coordinates[1];
					$lon = $data->Placemark[0]->Point->coordinates[0];	
				
					$radius = .3;
				
					if($this->layoutPath == 'json') :
						$this->set('restaurants', $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius)));					
					else :		
						$ids = $this->Restaurant->find('nearby', array('lat' => $lat, 'lon' => $lon, 'radius' => $radius, 'forPagination' => true));
						if($ids) :
							$ids = implode(', ', $ids);
							$where = 'Restaurant.id IN (' . $ids . ')';		
							$this->set('restaurants', $this->paginate('Restaurant', $where));
						else : 
							$this->set('restaurants', array());
						endif;
					endif;					
				endif;
		# whoa google fail	
			else :
				$this->Session->setFlash('Your address could not be found, double check and try again.', true, array('class' => 'error'));
				$this->redirect($this->referer());
			endif;
		
			$this->set('search_content', $this->params['named']['el']);
		endif;
	}

	function vegetarian() {
		if($this->layoutPath == 'json') :
			$this->set('restaurants', $this->Restaurant->find('all', array('conditions' => 'Restaurant.vegetarian = 1')));
		else :		
			$restaurants = $this->paginate('Restaurant','Restaurant.vegetarian = 1');
			if($restaurants) :
				$this->set('restaurants', $restaurants);
			else : 
				$this->set('restaurants', array());
			endif;
		endif;		
	}

	function vegan() {
		if($this->layoutPath == 'json') :
			$this->set('restaurants', $this->Restaurant->find('all', array('conditions' => 'Restaurant.vegan = 1')));
		else :		
			$restaurants = $this->paginate('Restaurant','Restaurant.vegan = 1');
			if($restaurants) :
				$this->set('restaurants', $restaurants);
			else : 
				$this->set('restaurants', array());
			endif;
		endif;		
	}
	
	function glutenfree() {
		if($this->layoutPath == 'json') :
			$this->set('restaurants', $this->Restaurant->find('all', array('conditions' => 'Restaurant.gluten_free = 1')));
		else :		
			$restaurants = $this->paginate('Restaurant','Restaurant.gluten_free = 1');
			if($restaurants) :
				$this->set('restaurants', $restaurants);
			else : 
				$this->set('restaurants', array());
			endif;
		endif;		
	}	
	
	function id2slug($id = null) {
		if(!$id) :
			$this->redirect(array('controller' => 'restaurants', 'action' => 'home'));
		endif;
		
		$restaurant = $this->Restaurant->read(null, $id);
		if(!empty($restaurant)) :
			$this->redirect('/' . $restaurant['Restaurant']['slug']);
		else :
			$this->redirect(array('controller' => 'restaurants', 'action' => 'home'));		
		endif;
	}

	// function rotating_logos() {
	// 	$this->Restaurant->recursive = -1;
	// 	$restaurants = $this->Restaurant->findAll('Restaurant.package_id = 3', null, 'RAND()', 7);
	//         if(isset($this->params['requested'])) {
	//              return $restaurants;
	//         }
	//         $this->set('restaurants', $restaurants);
	// }

	function admin_index() {
		$this->Restaurant->recursive = 0;
		$this->paginate['Restaurant']['limit'] = 25;
		$this->set('restaurants', $this->paginate());
	}

	function admin_view($id = null) {
		if (!$id) {
			$this->Session->setFlash(__('Invalid Restaurant', true));
		}
		$this->set('restaurant', $this->Restaurant->read(null, $id));
	}

	function admin_add() {
		if (!empty($this->data)) {
			$this->Restaurant->create();
			if ($this->Restaurant->save($this->data)) {
				$this->Session->setFlash('Restaurant saved.', true, array('class' => 'success'));
				$this->redirect(array('action' => 'index', 'admin' => true));				
			} else {
				$this->Session->setFlash('The Restaurant cound not be saved.', true, array('class'=>'error'));								
			}
		}
		
	# set a blank tag list, one from this->data, or one from the restaurants tags
		if((!isset($this->data['Restaurant']['taglist'])) && ((!isset($this->data['Tag'])) || (sizeof($this->data['Tag']) <= 0))) :
			$taglist = '';
		elseif(isset($this->data['Restaurant']['taglist'])) :
			$taglist = $this->data['Restaurant']['taglist'];
		else :
			$temptags = array();
			foreach($this->data['Tag'] as $tag) :
				$temptags[] = $tag['name'];
			endforeach;
			$taglist = implode(', ', $temptags);
		endif;		
		
	# all tags w/ weights
		$tags = $this->Tag->weights();
		$cuisines = $this->Restaurant->Cuisine->find('list');		
		$packages = $this->Restaurant->Package->find('list');
		$areas = $this->Restaurant->Area->find('list');
		$this->set(compact('tags', 'packages', 'areas', 'cuisines', 'taglist'));
	}

	function admin_edit($id = null) {		
		if (!$id && empty($this->data)) {
				$this->Session->setFlash('Invalid Restaurant.', true, array('class' => 'error'));			
			$this->redirect(array('action' => 'index', 'admin' => true));				
		}
		if (!empty($this->data)) {
			if ($this->Restaurant->save($this->data)) {
				$this->Session->setFlash('The Restaurant has been saved.', true, array('class' => 'success'));
				$this->redirect($this->referer());				
			} else {
				$this->Session->setFlash('The Restaurant cound not be saved.', true, array('class'=>'error'));				
			}
		}
		if (empty($this->data)) {
			$this->data = $this->Restaurant->read(null, $id);
		}
		
	# set a blank tag list, one from this->data, or one from the restaurants tags
		if((!isset($this->data['Restaurant']['taglist'])) && ((!isset($this->data['Tag'])) || (sizeof($this->data['Tag']) <= 0))) :
			$taglist = '';
		elseif(isset($this->data['Restaurant']['taglist'])) :
			$taglist = $this->data['Restaurant']['taglist'];
		else :
			$temptags = array();
			foreach($this->data['Tag'] as $tag) :
				$temptags[] = $tag['name'];
			endforeach;
			$taglist = implode(', ', $temptags);
		endif;		
		
	# all tags w/ weights
		$tags = $this->Tag->weights();
		$cuisines = $this->Restaurant->Cuisine->find('list');		
		$packages = $this->Restaurant->Package->find('list');
		$areas = $this->Restaurant->Area->find('list');
		$this->set(compact('tags','packages','areas', 'cuisines', 'taglist'));
	}

	function admin_delete($id = null) {
		if (!$id) {
			$this->Session->setFlash('Invalid Restaurant', true, array('class' => 'error'));
			$this->redirect(array('action' => 'index', 'admin' => true));							
		}
		if ($this->Restaurant->del($id)) {
			$this->Session->setFlash('Restaurant deleted', true, array('class' => 'success'));
			$this->redirect(array('action' => 'index', 'admin' => true));							
		}
	}
	
	function admin_renewal() {
		$this->Restaurant->recursive = 0;
		$this->paginate['Restaurant']['limit'] = 25;
		$this->paginate['Restaurant']['order'] = 'Restaurant.upgraded DESC';		
		$this->paginate['Restaurant']['conditions'] = array(
			'NOW() > (INTERVAL 11 Month + Restaurant.upgraded)' 
		);
		$this->set('restaurants', $this->paginate());		
		$restaurants = $this->paginate('Restaurant');
	} // admin_rewnewal	

	function admin_upgraded() {
		$this->Restaurant->recursive = 0;
		$this->paginate['Restaurant']['limit'] = 25;
		$this->paginate['Restaurant']['order'] = 'Restaurant.name ASC';		
		$this->paginate['Restaurant']['conditions'] = array(
			'Restaurant.package_id = 2'
		);
		$this->set('restaurants', $this->paginate());		
		$restaurants = $this->paginate('Restaurant');
	} // admin_rewnewal	

}
?>