<?php
class Restaurant extends AppModel {

	var $name = 'Restaurant';
	var $useTable = 'restaurants';
	var $actsAs = array('Sluggable' => array('label' => 'name'), 'Increment'=>array('incrementFieldName'=>'inaccurate'));
	var $order = 'Restaurant.name ASC';
	var $testing = false;

	//The Associations below have been created with all possible keys, those that are not needed can be removed
	var $belongsTo = array(
			'Package' => array('className' => 'Package',
								'foreignKey' => 'package_id',
								'conditions' => '',
								'fields' => '',
								'order' => ''
			),
			'Area' => array('className' => 'Area',
								'foreignKey' => 'area_id',
								'conditions' => '',
								'fields' => '',
								'order' => ''
			),
			'Cuisine' => array('className' => 'Cuisine',
								'foreignKey' => 'cuisine_id',
								'conditions' => '',
								'fields' => '',
								'order' => ''
			)			
	);

	var $hasOne = array(
			'Menu' => array('className' => 'Menu',
								'foreignKey' => 'restaurant_id',
								'dependent' => true,
								'conditions' => '',
								'fields' => '',
								'order' => '',
								'limit' => '',
								'offset' => '',
								'exclusive' => '',
								'finderQuery' => '',
								'counterQuery' => ''
			)
	);

	var $hasMany = array(
			'Picture' => array('className' => 'Picture',
								'foreignKey' => 'restaurant_id',
								'dependent' => true,
								'conditions' => '',
								'fields' => '',
								'order' => '',
								'limit' => '',
								'offset' => '',
								'exclusive' => '',
								'finderQuery' => '',
								'counterQuery' => ''
			),
			'Review' => array('className' => 'Review',
								'foreignKey' => 'restaurant_id',
								'dependent' => true,
								'conditions' => '',
								'fields' => '',
								'order' => '',
								'limit' => '',
								'offset' => '',
								'exclusive' => '',
								'finderQuery' => '',
								'counterQuery' => ''
			),
			'User' => array('className' => 'User',
								'foreignKey' => 'restaurant_id',
								'dependent' => true,
								'conditions' => '',
								'fields' => '',
								'order' => '',
								'limit' => '',
								'offset' => '',
								'exclusive' => '',
								'finderQuery' => '',
								'counterQuery' => ''
			)			
	);

	var $hasAndBelongsToMany = array(
			'Tag' => array('className' => 'Tag',
						'joinTable' => 'restaurants_tags',
						'foreignKey' => 'restaurant_id',
						'associationForeignKey' => 'tag_id',
						'unique' => true,
						'conditions' => '',
						'fields' => '',
						'order' => 'name ASC',
						'limit' => '',
						'offset' => '',
						'finderQuery' => '',
						'deleteQuery' => '',
						'insertQuery' => ''
			)		
	);
	
	public $admin_validate = array(
		'name' => array(
			'required' => array(
				'rule' => array('between', 2, 100),
				'required' => true,				
				'message' => 'Name must be between 2 to 100 characters'
			)
		),
		'street' => array(
			'required' => array(
				'rule' => array('between', 3, 100),
				'required' => true,				
				'message' => 'Street must be between 3 to 100 characters'				
			)
		), 
		'zip' => array(
			'zipcode' => array(
				'rule' => array('postal', null, 'us'),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Zipcode must be a valid zipcode'				
			)
		),	
		'phone' => array(
			'phone' => array(
				'rule' => array('phone', null, 'us'),				
				'required' => false,	
				'allowEmpty' => true,		
				'message' => 'Must be a valid phone number.'
			)
		),			
		'lat' => array(
			'numeric' => array(
				'rule' => array('decimal'),
				'required' => true,
				'message' => 'Lattitude must be numeric.'				
			)
		),
		'lon' => array(
			'numeric' => array(
				'rule' => array('decimal'),
				'required' => true,
				'message' => 'Longitude must be numeric.'				
			)
		),		
		'website' => array(
			'url' => array(
				'rule' => array('url', true),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Must be a valid website. Make sure you include "http://".'
			)
		),	
		'cost' => array(
			'numeric' => array(
				'rule' => 'numeric', 
				'required' => true,
				'message' => 'Cost must be numeric.'
			),
			'lessthan' => array(
				'rule' => array('comparison', '<=', 5),
				'message' => 'Cost must be a number 0 — 5'
			),
			'greaterthan' => array(
				'rule' => array('comparison', '>=', 0),
				'message' => 'Cost must be a number 0 — 5'
			)			
		),	
		'email' => array(
			'email' => array(
				'rule' => array('email', true),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Must be a valid email address.'
			)
		),
		'facebook' => array(
			'url' => array(
				'rule' => array('url', true),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Must be a valid facebook url. Make sure you include "http://".'
			)
		),
		'rating' => array(
			'numeric' => array(
				'rule' => 'numeric', 
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Rating must be numeric.'
			),
			'lessthan' => array(
				'rule' => array('comparison', '<=', 5),
				'message' => 'Rating must be a number 0 — 5'
			),			
			'greaterthan' => array(
				'rule' => array('comparison', '>=', 0),
				'message' => 'Rating must be a number 0 — 5'
			)			
		),				
	);

	public $validate = array(
		'phone' => array(
			'phone' => array(
				'rule' => array('phone', null, 'us'),				
				'required' => true,				
				'message' => 'Must be a valid phone number.'
			)
		),	
		'email' => array(
			'email' => array(
				'rule' => array('email', true),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Must be a valid email address.'
			)
		),		
		'website' => array(
			'url' => array(
				'rule' => array('url', true),
				'required' => false,
				'allowEmpty' => true,				
				'message' => 'Must be a valid website. Make sure you include "http://".'
			)
		),				
		'facebook' => array(
			'url' => array(
				'rule' => array('url', true),
				'required' => false,
				'allowEmpty' => true,
				'message' => 'Must be a valid facebook url. Make sure you include "http://".'
			)
		)		
	);
	
	function paginateCount($conditions, $recursive) {
    	return $this->find('count'
    				  	, array('conditions' => $conditions,
							  	'recursive' => $recursive, 
							  	'fields' => 'COUNT(DISTINCT Restaurant.id) AS count')); 	
	}

	function beforeSave() {
	# format the phone number
		if(isset($this->data['Restaurant']['phone'])) {
			$this->data['Restaurant']['phone'] = $this->formatPhone($this->data['Restaurant']['phone']);
		}
		
	# get rid of the upgrade date if not a premium package
		if($this->data['Restaurant']['package_id'] != 2) :
			$this->data['Restaurant']['upgraded'] = null;
		endif;
		
	# unset some things that aren't there
		// if(isset($this->data['Restaurant']['website']) == '') {
		// 	$this->data['Restaurant']['website'] = 'Not available';
		// }
		
	#format tags & save new tags
		if(isset($this->data['Restaurant']['taglist'])) :
			$tags = explode(',', $this->data['Restaurant']['taglist']);

			if(sizeof($tags) > 0) :
			$ids = array();
			foreach($tags as $tag) :
				$tag = trim($tag);
				if($tag != '') :
					$slug = $this->Behaviors->Sluggable->__slug($tag, $this->Behaviors->Sluggable->__settings['Tag']);
			
				# if the tag doesn't exit, add it	
					if(!$foundTag = $this->Tag->findBySlug($slug)) :				
						$this->Tag->data['Tag']['name'] = $tag;
						$this->Tag->data['Tag']['slug'] = $slug;
						if(!$this->Tag->save()) :
							// something
						endif;
						$ids[] = $this->Tag->id;
						unset($this->Tag->data);					
						$this->Tag->id = null;
					else :
						$ids[] = $foundTag['Tag']['id'];
					endif;				
				endif;
			endforeach;
			endif;

			$this->data['Tag']['Tag'] = $ids;
		endif;
		
		return true;
	}
	
	function afterSave() {
		$this->Tag->prune();
		return true;
	}
	
	function __findCuisine($options) {
		# get the tag
		$this->Cuisine->recursive = -1;
		$cuisine = $this->Cuisine->findBySlug($options['cuisine']);
		# remove tag from the options array so we can use it in find
		unset($options['cuisine']);

		# if you want to paginate through these return only the ids
		if(isset($options['forPagination']) && $options['forPagination']) :
			#find the restaurants with that cuisine
			$restaurants = $this->find('all', array('conditions' => 'cuisine_id = ' . $cuisine['Cuisine']['id']));
			$ids = Set::extract($restaurants, '{n}.Restaurant.id');		
		
			if(sizeof($ids)) :
				return $ids;
			else :
				return false;
			endif;
		endif;

		# otherwise return a big result set
		# set the where condition
		# find the restaurants with that area		
		$options['conditions'] = 'Restaurant.cuisine_id = ' . $cuisine['Cuisine']['id'];
		
		$this->recursive = -1;
		return $this->find('all', $options);	
	} // __findCuisine
	
	function __findRandom() {
		#find the restaurants with that tag
		$count = $this->find('count', array('order' => 'Restaurant.name ASC'));
		$rand = rand(0, $count);
		
		$restaurants = $this->find('all', array('order' => 'Restaurant.name ASC', 'limit' => 1, 'offset' => $rand));
		
		return $restaurants;
	} // __findTag	
		
	function __findTag($options) {
		# get the tag
		$tag = $this->Tag->findBySlug($options['tag']);
		# remove tag from the options array so we can use it in find
		unset($options['tag']);
		
		#find the restaurants with that tag
		$restaurants = $this->RestaurantsTag->find('all', array('conditions' => 'tag_id = ' . $tag['Tag']['id']));
		$ids = Set::extract($restaurants, '{n}.RestaurantsTag.restaurant_id');
	
		# if you want to paginate through these return only the ids
		if(isset($options['forPagination']) && $options['forPagination']) :
			if(sizeof($ids)) :
				return $ids;
			else :
				return false;
			endif;
		endif;

		# otherwise return a big result set
		# set the where condition
		if(sizeof($ids)) :
			$ids = implode(', ', $ids);
			$options['conditions'] = 'Restaurant.id IN (' . $ids . ')';
		endif;
	
		$this->recursive = -1;
		return $this->find('all', $options);
	} // __findTag
	
	function __findArea($options) {
		# get the tag
		$this->Area->recursive = -1;
		$area = $this->Area->findBySlug($options['area']);
		
		if(!$area) :
			return false;
		endif;
		
		# remove tag from the options array so we can use it in find
		unset($options['area']);

		# if you want to paginate through these return only the ids
		if(isset($options['forPagination']) && $options['forPagination']) :
			#find the restaurants with that cuisine
			$restaurants = $this->find('all', array('conditions' => 'area_id = ' . $area['Area']['id']));
			$ids = Set::extract($restaurants, '{n}.Restaurant.id');		
		
			if(sizeof($ids)) :
				return $ids;
			else :
				return false;
			endif;
		endif;

		# otherwise return a big result set
		# set the where condition
		# find the restaurants with that area		
		$options['conditions'] = 'Restaurant.area_id = ' . $area['Area']['id'];
		
		$this->recursive = -1;
		return $this->find('all', $options);
	} // __findArea
	
	function __findNearby($options) {
		$lat = $options['lat'];
		$lon = $options['lon'];
		$radius = $options['radius'];
		
		# unset the things
		unset($options['lat']); unset($options['lon']); unset($options['radius']);
		
		#find the restaurants	
		$options['conditions'] = "(SQRT(
			  (69.1 * (Restaurant.lat - $lat))*(69.1 * (Restaurant.lat - $lat)) +
			  (69.1 * (Restaurant.lon - $lon) * COS($lat/57.3))*(69.1 * (Restaurant.lon - $lon) * COS($lat/57.3))) < $radius)";		
			
		# if you want to paginate through these return only the ids
		if(isset($options['forPagination']) && $options['forPagination']) :
			#find the restaurants with that cuisine
			$restaurants = $this->find('all', $options);
			$ids = Set::extract($restaurants, '{n}.Restaurant.id');		
			if(sizeof($ids)) :
				return $ids;
			else :
				return false;
			endif;
		endif;			
			
		$this->recursive = -1;
		return $this->find('all', $options); 
	} // findNearby
	
	function __findPartial($options) {
		
		$conditions = 'Restaurant.hours = "" OR Restaurant.phone = "" OR Restaurant.website = ""';
		
		$options['conditions'] = $conditions;
		return $this->find('all', $options); 
	} // findNearby	
	
	function search($s) {
		
	# check for testing
		if($_SERVER['REMOTE_ADDR'] == '24.125.205.168' || $_SERVER['REMOTE_ADDR'] == '173.53.65.81') : 
			$this->testing = true;
		endif;	
		
		
	# --- TESTING ------------------------------------------------------	
		if(true) :		
		//if(false) :	
		
		# check for blank search
			if(trim($s) == '') :
				$restaurants = $this->find('all');

				//unpack the id's into a better array
				$ids = null;
				if(sizeof($restaurants) > 0) :
					$ids = array();
					foreach($restaurants as $item) :
						$ids[] = $item['Restaurant']['id'];
					endforeach;
					return $ids;
				endif;		
			endif;
		
		# intelligent explosion -- TO COME
			$searchTerms = explode(' ', $s);			
		
		# handle 's			
			foreach($searchTerms as $term) :
				if((substr(strtolower($term), -1) == 's') && substr(strtolower($term), -2) != "'s") :
					$searchTerms[] = strtolower(Sanitize::escape(substr($term, 0, -1) . "'s"));
				endif;
			endforeach;		
		
		# remove common words	
			foreach($searchTerms as $key => $term) :
				if($term == 'the' || $term == 'is' || $term == 'a' || trim($term) == '') :
					unset($searchTerms[$key]);
				endif;				
			endforeach;		
			
		# get tag list for each term
			$taglist = array();
			$taglist['__all'] = array();
			foreach($searchTerms as $term) :
				$tagSearch = "Tag.name LIKE '%".$term."%'";
				$this->Tag->recursive = -1;
				$tags = $this->Tag->find('all', array('conditions' => $tagSearch));			
				$ids = Set::extract($tags, '{n}.Tag.id');	
				if($ids) :			
					$taglist[$term] = $ids;
					$taglist['__all'] = array_merge($taglist['__all'], $ids);
				endif;
			endforeach;
			
			$sql = '';			
			
		# direct matches score a ton of points
			$sql = '
				SELECT DISTINCT Restaurant.name, Restaurant.id as restaurant_id, 4 as weight
				FROM restaurants AS Restaurant
				WHERE (
					Restaurant.name LIKE "%'.$s.'%")';			
			
		# Search for names first, with a higher weight
			foreach($searchTerms as $term) :
				$sql .= '
		UNION ALL ';
	
				$sql .= '
					SELECT DISTINCT Restaurant.name, Restaurant.id as restaurant_id, 3 as weight
					FROM restaurants AS Restaurant
					WHERE (
						Restaurant.name LIKE "%'.$term.'%")';
			endforeach;	
		
		# Search for Name AND others	
				if(sizeof($searchTerms) > 1) :	
				
					$sql .= '
		UNION ALL

						SELECT DISTINCT Restaurant.name, Restaurant.id as restaurant_id, 2 as weight
						FROM restaurants AS Restaurant
						LEFT JOIN cuisines AS Cuisine ON ( Restaurant.cuisine_id = Cuisine.id )
						LEFT JOIN areas AS Area ON ( Restaurant.area_id = Area.id )
						LEFT JOIN restaurants_tags AS RestaurantsTag ON ( RestaurantsTag.restaurant_id = Restaurant.id )
						WHERE (';
			
					$searchTerms2 = $searchTerms;
					$tmpArray = array();
					foreach($searchTerms as $term) :
				 		$tmp = '';
						$tmp = 'Restaurant.name LIKE "%' . $term . '%" AND (';
						foreach($searchTerms2 as $term2) :
							if($term2 != $term) :
								$areas[] = 'Area.name LIKE "%' . $term2 . '%"';
								$cuisines[] = 'Cuisine.name LIKE "%' . $term2 . '%"';
								if(isset($taglist[$term2]) && sizeof($taglist[$term2])) :
									$tmpTags[] = 'RestaurantsTag.tag_id IN (' . implode(', ', $taglist[$term2]) . ')';
								endif;
							endif;
						endforeach;

						if(isset($areas)) :
							$tmpArray[] = $tmp . implode(' OR ', $areas) . ')';
						endif;

						if(isset($cuisines)) :
						$tmpArray[] = $tmp . implode(' OR ', $cuisines) . ')';	
						endif;

						if(isset($tmpTags)) :
							$tmpArray[] = $tmp . implode(' OR ', $tmpTags) . ')';	
						endif;

						unset($areas);
						unset($cuisines);
						unset($tmpTags);
						reset($searchTerms2);					
					endforeach;
				
					if(sizeof($tmpArray)) :
						$sql .= implode("\n OR ", $tmpArray) . ')/*@@*/';
					else :
						$sql .= "false)";
					endif;
				endif;
		 	

		# Search for OR's 
			$counter = 0;
			foreach($searchTerms as $term) : 
				$counter ++;
				if(($counter > 1) || $sql != '') :
					$sql .= '
		UNION ALL ';
				endif;			
			
				$sql .= '
					SELECT DISTINCT Restaurant.name, Restaurant.id as restaurant_id, 1 as weight
					FROM restaurants AS Restaurant
					LEFT JOIN cuisines AS Cuisine ON ( Restaurant.cuisine_id = Cuisine.id )
					LEFT JOIN areas AS Area ON ( Restaurant.area_id = Area.id )';
				
				if(isset($taglist[$term]) && sizeof($taglist[$term])) :
					$sql .= '
						LEFT JOIN restaurants_tags AS RestaurantsTag ON ( RestaurantsTag.restaurant_id = Restaurant.id )';
				endif;

				$sql .= '
					WHERE (
						   Area.name LIKE "%'.$term.'%"
						OR Cuisine.name LIKE "%'.$term.'%"';
				if(isset($taglist[$term]) && sizeof($taglist[$term])) :
					$sql .= ' 
							OR RestaurantsTag.tag_id IN (' . implode(', ', $taglist[$term]) . ')';					
				endif;
			
				$sql .= '
					) ';
			endforeach;
		
		
			$sql .= 'ORDER BY weight DESC, name';
			
			$sql = 'SELECT *, SUM(weight) as score FROM (' . $sql .') as sq1
					GROUP BY restaurant_id
					ORDER BY score DESC, name ASC';
		
			//debug($sql);
			$results = $this->query($sql);
		
			//unpack the id's into a better array
			$ids = null;
			if(sizeof($results) > 0) :
				$ids = array();
				foreach($results as $item) :
					$ids[] = $item['sq1']['restaurant_id'];
				endforeach;
			endif;
		
			return $ids;
	# ---- LIVE ------------------------------------------------------------		
		else :
			$searchTerms = explode(' ', $s);		
		
			$sql = '
				SELECT sq1.* 
				FROM (
				';

			$counter = 0;
			foreach($searchTerms as $term) : 
				//get tags
				$tagSearch = "Tag.name LIKE '%".$term."%'";
				$this->Tag->recursive = -1;
				$tags = $this->Tag->find('all', array('conditions' => $tagSearch));
				$ids = Set::extract($tags, '{n}.Tag.id');
				$ids = implode(', ', $ids);
		
				$counter ++;
				if($counter > 1) :
					$sql .= '
						INNER JOIN (';
				endif;			
			
				$sql .= '
					SELECT DISTINCT Restaurant.id as restaurant_id
					FROM restaurants AS Restaurant
					LEFT JOIN cuisines AS Cuisine ON ( Restaurant.cuisine_id = Cuisine.id )
					LEFT JOIN areas AS Area ON ( Restaurant.area_id = Area.id )';
				
				if(strlen($ids) > 0) :
					$sql .= '
						LEFT JOIN restaurants_tags AS RestaurantsTag ON ( RestaurantsTag.restaurant_id = Restaurant.id )';
				endif;

				$sql .= '
					WHERE (
						Restaurant.name LIKE "%'.$term.'%"
						OR Area.name LIKE "%'.$term.'%"
						OR Cuisine.name LIKE "%'.$term.'%"';
				if(strlen($ids) > 0) :
					$sql .= ' 
							OR RestaurantsTag.tag_id IN (' . $ids . ')';					
				endif;
			
				$sql .= '
					)
				) as sq'.$counter;
			
				if($counter > 1) :
					$sql .= '
						ON sq'.($counter - 1).'.restaurant_id = sq'.$counter.'.restaurant_id';
				endif;
			endforeach;
		
			print_r($sql);
			$results = $this->query($sql);
		
			//unpack the id's into a better array
			$ids = null;
			if(sizeof($results) > 0) :
				$ids = array();
				foreach($results as $item) :
					$ids[] = $item['sq1']['restaurant_id'];
				endforeach;
			endif;
		
			return $ids;
		endif;
	}
	
	function formatPhone ($phone) {
	    $phone = preg_replace("/[^0-9]/", "", $phone);

	    if(strlen($phone) == 7)
		    return preg_replace("/([0-9]{3})([0-9]{4})/", "$1-$2", $phone);
	    elseif(strlen($phone) == 10)
		    return preg_replace("/([0-9]{3})([0-9]{3})([0-9]{4})/", "$1-$2-$3", $phone);
	    else
		    return $phone;
	} // format Phone


}
?>