<?php
class Tag extends AppModel {

	var $name = 'Tag';
	var $useTable = 'tags';
	var $actsAs = array('Sluggable' => array('label' => 'name'));	
	var $order = 'Tag.name ASC';

	//The Associations below have been created with all possible keys, those that are not needed can be removed
	var $hasAndBelongsToMany = array(
			'Restaurant' => array('className' => 'Restaurant',
						'joinTable' => 'restaurants_tags',
						'foreignKey' => 'tag_id',
						'associationForeignKey' => 'restaurant_id',
						'unique' => '',
						'conditions' => '',
						'fields' => '',
						'order' => '',
						'limit' => '',
						'offset' => '',
						'finderQuery' => '',
						'deleteQuery' => '',
						'insertQuery' => ''
			)
	);
	
	public $validate = array(
		'name' => array(
			'required' => array(
				'rule' => array('between', 3, 100),				
				'required' => true,	
				'allowEmpty' => false,
				'message' => 'Name must be between 3 to 100 characters'
			)			
		)
	);	
	
	function weights() {
		$sql = '
			SELECT Tag.*, count(*) as count
			FROM restaurants_tags as RestaurantTag
			LEFT JOIN tags as Tag ON Tag.id = RestaurantTag.tag_id
			GROUP BY tag_id
			ORDER BY Tag.name';
		$results = $this->query($sql);
		
		if(sizeof($results) > 0) :
			$tags = array();
			foreach($results as $tag) :
				$tag['Tag']['count'] = $tag[0]['count'];
				unset($tag[0]);		
				$tags[] = $tag;
			endforeach;
			return $tags;
		else :
			return false;
		endif;
	} // weights
	
	function prune() {
		$sql = '
			SELECT Tag . * , RestaurantsTag . *
			FROM tags AS Tag
			LEFT JOIN restaurants_tags AS RestaurantsTag ON Tag.id = RestaurantsTag.tag_id
			WHERE RestaurantsTag.tag_id IS NULL
			LIMIT 0 , 30';
		$results = $this->query($sql);
		
		if(sizeof($results)) :
			$tagsToDelete = array();
			foreach($results as $tag) :
				$tagsToDelete[] = $tag['Tag']['id'];
			endforeach;
			$conditions[] = 'Tag.id IN (' . implode(', ', $tagsToDelete) . ')';
			$this->deleteAll($conditions, false);
		endif;
	} // prune
}
?>